/***************************************************************************
 *   Copyright (C) 2005-2007 by Lisandro Damián Nicanor Pérez Meyer        *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QDebug>
#include <float.h>
#include "matrix.h"

Matrix::Matrix( int dimension )
{
	rowCount = dimension;
	columnCount = dimension;
	array = new double[rowCount * columnCount];

	/* We fill the matrix with -1 * DBL_MAX */
	int i;

	for( i=0; i<( rowCount * columnCount ); i++ )
		array[i] = -1 * DBL_MAX;
}

Matrix::Matrix( int columns, int rows )
{
	rowCount = rows;
	columnCount = columns;
	array = new double[rowCount * columnCount];

	/* We fill the matrix with -1 * DBL_MAX */
	int i;
	for( i=0; i<( rowCount * columnCount ); i++ )
		array[i] = -1 * DBL_MAX;
}

Matrix::~Matrix()
{
  delete[] array;
}

double Matrix::getValue(int row, int column) const
{
  return array[(row * columnCount) + column];
}

void Matrix::setValue(int row, int column, double value)
{
  array[(row * columnCount) + column] = value;
}

int Matrix::getRows() const
{
	return rowCount;
}

int Matrix::getColumns() const
{
	return columnCount;
}

void Matrix::debug() const
{
	for(int i=0; i<rowCount; i++)
	{
		for(int j=0; j<columnCount; j++)
		{
			printf(" %f ", getValue(i,j));
		}
		printf("\n");
	}
}

