/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SCENE
#define SCENE

#include <QWidget>
#include <QImage>
#include "graphicprimitive.h"
#include "matrix4x4.h"

// Forward declarations
class QPainter;
class QPaintEvent;

/// Scene provides the basics for drawing scenes
/**
	The Scene class holds a QImage that will show the current scene on it.
*/

class Scene : public QWidget
{
	Q_OBJECT

public:
	Scene(QWidget * parent = 0);

	~Scene();

	/// Draw the figures inside the scene
	void draw();

	/// Add a cube
	void addCube();

public slots:

	/// Enable / disable perspective proyection
	void perspectiveProyection(int state);

	/* Figures */
	/// When a figure is selected, do the appropiate
	void figureSelected(int figure);

	/// Change the selected figure x displacement
	void changeXDisplacement(int displacement);

	/// Change the selected figure y displacement
	void changeYDisplacement(int displacement);

	/// Change the selected figure z displacement
	void changeZDisplacement(int displacement);

	/// Change the selected figure x rotation angle
	void changeAngleOfRotationX(int angle);

	/// Change the selected figure y rotation angle
	void changeAngleOfRotationY(int angle);

	/// Change the selected figure z rotation angle
	void changeAngleOfRotationZ(int angle);

	/// Change the selected figure x scale
	void changeXScale(int scale);

	/// Change the selected figure y scale
	void changeYScale(int scale);

	/// Change the selected figure z scale
	void changeZScale(int scale);

	/* Scene */
	/// Change the scene x displacement
	void changeSceneXDisplacement(int displacement);

	/// Change the scene y displacement
	void changeSceneYDisplacement(int displacement);

	/// Change the scene z displacement
	void changeSceneZDisplacement(int displacement);

	/// Change the scene x rotation angle
	void changeSceneAngleOfRotationX(int angle);

	/// Change the scene y rotation angle
	void changeSceneAngleOfRotationY(int angle);

	/// Change the scene z rotation angle
	void changeSceneAngleOfRotationZ(int angle);

	/// Change the scene x scale
	void changeSceneXScale(int scale);

	/// Change the scene y scale
	void changeSceneYScale(int scale);

	/// Change the scene z scale
	void changeSceneZScale(int scale);

signals:

	/// Send the parameters
	void changeParameters(double xDisplacement, double yDisplacement,
	                      double rotationAngleX, double rotationAngleY,
	                      double rotationAngleZ, double xScale, double yScale);

protected:
	/// Paints the scene in an event
	void paintEvent(QPaintEvent * event);

private:
	/// Get the projection matrix
	Matrix4X4 getProjectionMatrix();

	/// Get the scale matrix
	Matrix4X4 getScaleMatrix();

	/// Get the rotation matrix
	Matrix4X4 getRotationMatrix();

	/// Get the translation matrix
	Matrix4X4 getTranslationMatrix();

	/// The image in which we are going to draw
	QImage * image;

	/// Perspective proyection status
	bool perspective;

	/// List of figures / obects
	QList< GraphicPrimitive * > figures;
	/// Holds the ID of the selected figure
	int selectedFigure;

	/// Scene x displacement
	double xDisplacement;
	/// Scene y displacement
	double yDisplacement;
	/// Scene z displacement
	double zDisplacement;

	/// Scene angle of rotation around the x axis
	double xAngle;
	/// Scene angle of rotation around the y axis
	double yAngle;
	/// Scene angle of rotation around the z axis
	double zAngle;

	/// Scene scale in x
	double xScale;
	/// Scene scale in y
	double yScale;
	/// Scene scale in z
	double zScale;
};

#endif // Scene








